#!/bin/sh
###############################################################################
##
## support.dell.com Instructions:
##
##   To download this file go back to the previous page, Right-Click on the
##   download link and use the "Save Target As" file option.
##
###############################################################################

###############################################################################
##
##              DELL INC. PROPRIETARY INFORMATION
##
##  This software is supplied under the terms of a license agreement or
##  nondisclosure agreement with Dell Inc. and may not be copied or disclosed 
##  except in accordance with the terms of that agreement.
##
##  Copyright (c) 2003-2010 Dell, Inc. All Rights Reserved.
## 
##  Abstract/Purpose:
##
##  Environment:
##    Linux
##
##  Version:
##    $Revision $
##
###############################################################################
# Implement script security, only on Linux systems
#  - sanitize IFS
#  - insure 'unalias' is not a funcion
#  - unset all aliases and quote unalias so it is not alias-expanded
#  - insure 'command' is not a function
#  - sanitize PATH 
[ "$(uname -s)" = "Linux" ] && {
#    IFS=$' \t\n'
    IFS=`printf ' \t\n+'`; IFS=${IFS%+}
    unset -f unalias
    \unalias -a
    unset -f command
    SYSPATH="$(command -p getconf PATH 2>/dev/null)"
    [ -z "$SYSPATH" ] && SYSPATH="/usr/bin:/bin"
    PATH="$SYSPATH:/sbin:/usr/sbin"
}

# Set umask
umask 0022

# Global constants - readonly variables
_TRUE=0 
_FALSE=1
_CMDOPTS="$(printf "%s" "$*")"

# Interactive command line flags and boolean functions
_VAL_NO_TERMINAL=$_FALSE
bNoTerminal() {
    [ $_VAL_NO_TERMINAL -eq $_TRUE ]
}

# Determine if the application has been launched from an xterm or XWindows
case $TERM in 
    # rxvt is a background terminal for Novell
    # ansi, vt100, screen are standard display terminals
    rxvt* | ansi* | vt100* | screen*) # Do nothing
        ;;
    dumb* | linux*)
        [ -n "$DISPLAY" -a -n "$XAUTHORITY" ] && {
            tty --quiet || exec xterm +ls -e $0 $@ || $0 $@
        }
        ;;
    # This will cause interactive DUPs to re-launch in an xterm window 
    # if stdin is not a terminal
    xterm*) 
        tty --quiet || _VAL_NO_TERMINAL=$_TRUE
        ;;
    *) exec xterm +ls -e $0 $@
        ;;
esac


## constants
_CURRDIR=$PWD

_EXIT_DELAY=1 # gives users a chance to read output on temporary terminals 

# Exit codes
_EXECUTION_SUCCESS=0
_EXECUTION_UNSUCCESS=1
_REBOOT_REQUIRED=2
_DEP_SOFT_ERROR=3
_DEP_HARD_ERROR=4
_QUAL_HARD_ERROR=5
_REBOOTING_SYSTEM=6
_SECURITY_ERROR=9
_UPGRADE_INFO=20
_UPDATE_SUCCESSFUL_SOFT_DEPENDENCY_NOT_MET=13
_REBOOT_REQUIRED_SOFT_DEPENDENCY_NOT_MET=14

_INTERNAL_HARD_DEPENDENCY=1
_INTERNAL_SOFT_DEPENDENCY=2
# Miscellaneous
_VERBOSE=1
ROOT_ID=`id -ru`
# Only users with $UID 0 have root privileges.
isRoot() {
    [ $ROOT_ID -eq 0 ]
}
_DKS_BASE_DIR=/var/omsa/dks
_TARGET_DIR="$PWD"
# width for console text output formatting
_WIDTH=100

# required files - constant
_TMPFILE_PREFIX="/tmp/.dellSP"
_SPUTILITY_BIN="./sputility.bin"
_MESSAGE_FILE="en.prop"
_LOCK_FILE="/var/lock/.spsetup"
_IC_LOCK_FILE="/var/lock/invcol.tmp";
_VERSION_FILE="$_TARGET_DIR"/Version.txt
_HELP_FILE="$_TARGET_DIR"/sphelp.txt
_PACKAGE_XML_FILE="package.xml"
_PIE_CONFIG_SH_FILE="./PIEConfig.sh"
_INVENTORY_XML_FILE="inv.xml"
_BUILDVER="./buildVer.sh"
_PAYLOAD_DIR="payload"
_DONOTMOVE="donotmovepayload"
_UDEVHAL_SHFILE="./UdevHal.sh"
_UDEVHAL_SHFILE_ARGSDISABLE="disablepopup"
_UDEVHAL_SHFILE_ARGSENABLE="enablepopup"
_IDRAC_POPUP_TMP_FILE="idracpopupchecktmpfile.$$"
_DEP_ERROR_TMP_FILE="deperrtxtfile.$$"
_DUPLOGNAME="./dupname.sh"
_SUPPORT_LOG_FILE=""
# include DUP framework version, Copyright info, build number
. $_BUILDVER

# log folders - constant
_COMPLETE_BASE_DIR="/var/log"
_LOG_DIR="$_COMPLETE_BASE_DIR/dell/updatepackage/log"
_COMPLETE_LOG_DIR="$_LOG_DIR/support"
_LOG_DIR_DELL="/dell"
_LOG_DIR_UPDATEPACKAGE="/updatepackage"
_LOG_DIR_LOG="/log"
_LOG_DIR_SUPPORT="/support"
_SMAL_DEBUG_LOG_FILE="./SMAL_DBG.log"
_DEBUG_LOG_FILE="./debug.log"
# Initialize Support Log Size and Limit
_SUPPORT_LOG_SIZE_LIMIT=500000 # 500 K

# XML Headings and tags
_XML_HEADING_PATTERN="<?xml version=.*?>"
_XML_HEADING="<?xml version=\"1.0\" encoding=\"UTF-8\"?>"

# logging booleans
_IS_LOGGING=$_FALSE
_MAKE_SYSLOG_ENTRY=$_FALSE
exitDUP () {
	returnCode=$1
	if [ ! -z $_SUPPORT_LOG_FILE ]
	then
		if [ $returnCode -ne $_EXECUTION_SUCCESS -a $returnCode -ne $_REBOOTING_SYSTEM -a $returnCode -ne $_REBOOT_REQUIRED -a $returnCode -ne $_DEP_SOFT_ERROR ]
			then
			if [ -f $_SMAL_DEBUG_LOG_FILE -a -f $_SUPPORT_LOG_FILE ]
			then
				echo "=====SMAL Debug Log Starts=====" >> $_SUPPORT_LOG_FILE 2>/dev/null
				cat $_SMAL_DEBUG_LOG_FILE >> $_SUPPORT_LOG_FILE 2>/dev/null
				echo "=====SMAL Debug Log Ends=====" >> $_SUPPORT_LOG_FILE 2>/dev/null
			fi
			if [ -f $_DEBUG_LOG_FILE -a -f $_SUPPORT_LOG_FILE ]
			then
				echo "=====Debug Log Starts=====" >> $_SUPPORT_LOG_FILE 2>/dev/null
				cat $_DEBUG_LOG_FILE >> $_SUPPORT_LOG_FILE 2>/dev/null
				echo "=====Debug Log Ends=====" >> $_SUPPORT_LOG_FILE 2>/dev/null
			fi
		fi
	fi
	exit $returnCode
}
bLogging() {
    [ $_IS_LOGGING -eq $_TRUE ]
}
bMakeSysLogEntry() {
    [ $_MAKE_SYSLOG_ENTRY -eq $_TRUE ]
}

# lock file booleans
_MADELOCK=$_FALSE
bLockMade() {
    [ $_MADELOCK -eq $_TRUE ]
}
_FOUNDLOCK=$_FALSE
bLockFound() {
    [ $_FOUNDLOCK -eq $_TRUE ]
}
_FOUNDICLOCK=$_FALSE
bICLockFound() {
    [ $_FOUNDICLOCK -eq $_TRUE ]
}
# Force and tiered force variables
_DO_EXECUTE_UPDATE=$_TRUE
_TARGET_COMPARE_ID="TargetCompareState"
_TARGET_COMPARE_STATE=""
_COMPONENT_TYPE_ID="componentType"
_COMPONENT_TYPES=""
_SOFTDEP_CHECK_INFO=""
_HARDDEP_CHECK_INFO=""
iRCDepCheck=99
# Command Line flags and boolean functions
_VAL_VERSION=$_FALSE
_VAL_DEBUGINV=$_FALSE
_VAL_XML=$_FALSE
_DEP_WARNING=$_FALSE
_VAL_QUIET=$_FALSE
_VAL_ALLOW_INSECURE_EXEC=$_FALSE
_VAL_CHECK_ONLY=$_FALSE
_VAL_FORCE=$_FALSE
_FORCE_LEVEL=-1
_VAL_REBOOT=$_FALSE
_APPEND_SUPPORT=$_FALSE
# In DUP 2.0, 64-bit platform blocking is not in effect.
_TEST_FOR_64_BIT_PLATFORMS=$_FALSE
bVersion() {
    [ $_VAL_VERSION -eq $_TRUE ]
}
bDebugInv() {
    [ $_VAL_DEBUGINV -eq $_TRUE ]
}
bXml() {
    [ $_VAL_XML -eq $_TRUE ]
}
bDependecyWarning() {
        [ $_DEP_WARNING -eq $_TRUE ]
}
bQuiet() {
    [ $_VAL_QUIET -eq $_TRUE ]
}
bAllowInsecureExec() {
        [ $_VAL_ALLOW_INSECURE_EXEC -eq $_TRUE ]
}
bCheckOnly() {
    [ $_VAL_CHECK_ONLY -eq $_TRUE ]
}
bForce() {
    [ $_VAL_FORCE -eq $_TRUE ]
}
bTForce() {
    [ $_FORCE_LEVEL -ne -1 ]
}
bReboot() {
    [ $_VAL_REBOOT -eq $_TRUE ]
}
bAppendSupport() {
    [ $_APPEND_SUPPORT -eq $_TRUE ]
}
bTestFor64BitPlatforms() {
    [ $_TEST_FOR_64_BIT_PLATFORMS -eq $_TRUE ]
}

# Translate exit code to result string
bSuccess() {
    iRc=$1
    [ $_EXECUTION_SUCCESS -eq $iRc -o \
      $_REBOOT_REQUIRED -eq $iRc -o \
      $_REBOOTING_SYSTEM -eq $iRc -o \
      $_DEP_SOFT_ERROR -eq $iRc ]
}

# Translate Execution exit code to result string
bExecutionSuccess() {
    iRc=$1
    [ $_EXECUTION_SUCCESS -eq $iRc -o \
      $_REBOOT_REQUIRED -eq $iRc -o \
      $_REBOOTING_SYSTEM -eq $iRc ]
}

getResult() {
    iRc=$1
    if bSuccess $iRc; then
        echo success
    else
        echo fail
    fi
}

# Prints an unformatted message without lookup
# from message property file
msgSecure() {
    opts=""
    [ "$(printf "%s" )" != "-e" ] && opts="-e"
    line=""
    for arg in "$@"; do
        line=$line$arg
    done
    echo $opts "$line"
}
# Generic printf messaging
_msg() {
    search=$1
    shift
    entry=$(look -t= $search= $_MESSAGE_FILE)
    bFound=$?
    format="Message not found for id: $search"
    if [ $bFound -eq 0 ]; then
        format=$(printf "%s"  "$entry" | sed "s/^$search=\\(.*\\)/\\1/g")
    fi
    # insert %s for empty string to avoid printf error if format begins with -
    printf "%s$format" "" "$@"
}

# Prints a formatted message
msgFmt() {
    _msg "$@" | fmt -s -w $_WIDTH
}

# Prints a formatted message with "<Scriptname>:  " prefix.
msgScript() {
    message=$(echo "$_SCRIPTNAME:  ") 
    message=$message$(_msg "$@")
    echo $message | fmt -s -w $_WIDTH
}

# Support logging functions
logSupport() {
    message=$(msgFmt "$@")
    message="${message}"
    if [ "$PMBrandStatus" = "DEBRAND" ]; then
        echo "$message"|sed "s:Dell ::g" >> $_SUPPORT_LOG_FILE
    else
        echo "$message"|sed "s:Dell:$PMManufacturerName:g" >> $_SUPPORT_LOG_FILE
    fi

}

# Parses messages returned by sputility.bin -m
_APP_ARGS=appargs
_parseMessages() {
    RANDOM_NUM=`hexdump -n 2 -e '/2 "%u"' /dev/urandom`
    tmpFile=$(mktemp $_TMPFILE_PREFIX-messages$$-${RANDOM_NUM}.XXXXXX) || {
        echo "$@"
        return
    }
    li=""
    for li in "$@"; do
        if echo "$li" | grep "^$_APP_ARGS=" > /dev/null ; then
           if [ "$PMBrandStatus" = "DEBRAND" ]; then
              v=$(echo $li | sed "s/^$_APP_ARGS=\\(.*\\)/\\1/g;" | sed "s:Dell::g")
           else
              v=$(echo $li | sed "s/^$_APP_ARGS=\\(.*\\)/\\1/g;" | sed "s:Dell:$PMManufacturerName:g")
           fi
           eval msgFmt VERSION_UPDATE_INFO_DETAIL $v >> $tmpFile
        else
           if [ "$PMBrandStatus" = "DEBRAND" ]; then
               v=$(echo  $li | sed "s:Dell::g")
           else
               v=$(echo  $li | sed "s:Dell:$PMManufacturerName:g")
           fi
           echo $v >> $tmpFile
        fi
    done
    # scrub out Target compare values
    cat $tmpFile | egrep -v "^${_TARGET_COMPARE_ID}=|^${_COMPONENT_TYPE_ID}="
    rm -f $tmpFile
}

# Prints a formatted message from an output file produced from sputility.bin -m
_SED_QUOTE_LINES='s,",\\",g; s,^ *\(.*\) *$,"\1",g;'
printFormattedMessage() {
    # Scrub out appArgs display from console display
    if [ "$PMBrandStatus" = "DEBRAND" ]; then
        lines=$(cat $_MESSAGE_RESULT_FILE | grep -v $_APP_ARGS | sed "$_SED_QUOTE_LINES" |sed "s:Dell::g")
    else
        lines=$(cat $_MESSAGE_RESULT_FILE | grep -v $_APP_ARGS | sed "$_SED_QUOTE_LINES" |sed "s:Dell:$PMManufacturerName:g")
    fi
    eval _parseMessages $lines | fmt -s -w $_WIDTH
    displayDepInfo "$_INTERNAL_HARD_DEPENDENCY"
}

# Prints a the version update message from an output file produced from sputility.bin -m
printVersions() {
     lines=$(cat $_VALIDATION_MESSAGE_RESULT_FILE | grep $_APP_ARGS | sed "$_SED_QUOTE_LINES")
    eval _parseMessages $lines
}

# Prints a formatted message to the support log
logSupportFormattedMessage() {
    if [ "$PMBrandStatus" = "DEBRAND" ]; then
        lines=$(cat $_MESSAGE_RESULT_FILE | sed "$_SED_QUOTE_LINES" |sed "s:Dell::g")
    else
        lines=$(cat $_MESSAGE_RESULT_FILE | sed "$_SED_QUOTE_LINES" |sed "s:Dell:$PMManufacturerName:g")
    fi
    eval _parseMessages $lines >> $_SUPPORT_LOG_FILE
    if [ "$iRCDepCheck" -eq "$_INTERNAL_HARD_DEPENDENCY" ]; then
       logDisplayDepInfo "$_INTERNAL_HARD_DEPENDENCY"
    fi
}

logDisplayDepInfo() {
	displayDepInfo $1 >> $_SUPPORT_LOG_FILE
 	if [ -f "$_DEP_ERROR_TMP_FILE" ]; then
       	   cat "$_DEP_ERROR_TMP_FILE" >> $_SUPPORT_LOG_FILE
           rm -f "$_DEP_ERROR_TMP_FILE" > /dev/null 2>&1
        fi
	if [ "$iRCDepCheck" -eq "$_INTERNAL_HARD_DEPENDENCY" ]; then
	   exitDUP $_DEP_HARD_ERROR
	fi
}

# Prints a formatted update error message to stdout
# using svmExeMsg.xsl to parse the xml result file.
printUpdateErrors() {
    which xsltproc > /dev/null 2>&1 && {
        xsltproc svmExeMsg.xsl $_XML_RESULT_FILE |sed '/^Device: $/d;/^  Application: $/d' | fmt -s -w $_WIDTH
        return
    }
    # default: original implementation of printing messages
    printFormattedMessage 
}

# Prints a formatted update error message to the support log 
# using svmExeMsg.xsl to parse the xml result file.
logSupportUpdateErrors() {
    which xsltproc > /dev/null 2>&1 && {
        xsltproc svmExeMsg.xsl $_XML_RESULT_FILE > /tmp/dupsup.log
        if [ "$PMBrandStatus" = "DEBRAND" ]; then
	   cat /tmp/dupsup.log|sed "s:Dell ::g" >>$_SUPPORT_LOG_FILE
	else
	   cat /tmp/dupsup.log|sed "s:Dell:$PMManufacturerName:g" >>$_SUPPORT_LOG_FILE
	fi
	rm -rf /tmp/dupsup.log
        return
    }
    # default: original implementation of logging messages
    logSupportFormattedMessage
}

# Prints an XML file, formatted via xmllint or 
# prints the XML file via 'cat' if xmllint is not in the path or
# xmllint parses a bad xml file
printXml() {
    if [ "$PMBrandStatus" = "DEBRAND" ]; then
       cat $1|sed "s:Dell ::g" >/tmp/duptmp.xml
    else
       cat $1|sed "s:Dell:$PMManufacturerName:g" >/tmp/duptmp.xml
    fi
    which xmllint > /dev/null 2>&1 && xmllint --format /tmp/duptmp.xml 2>/dev/null && rm -rf /tmp/duptmp.xml && return
    cat /tmp/duptmp.xml 2>/dev/null
	rm -f /tmp/duptmp.xml >/dev/null 2>&1
}

# Prints a package XML file, formatted via xmllint or
# prints the package XML file via 'cat' if xmllint fails
# xmllint throws away standard error; silently failing encoding errors.
printPackageXml() {
    if [ "$PMBrandStatus" = "DEBRAND" ]; then
       #1st sed is to remove Dell
       #2nd sed is to remove dell from dellVersion tag
       #3rd sed is to remove dell support site url
       cat $1|sed "s:Dell ::g"|sed "s:dell::g"|sed "s:PowerEdge::g"|sed "s:Power Edge::g"|sed "s:PowerVault::g"|sed "s:Power Vault::g"|sed "s:Equallogic::g" |sed "s:URL=\"http[^>]*::g" >/tmp/duptmp.xml
    else
       cat $1|sed "s:Dell:$PMManufacturerName:g"|sed "s:dell::g"|sed "s:PowerEdge::g"|sed "s:Power Edge::g"| sed "s:PowerVault::g"|sed "s:Power Vault::g"|sed "s:Equallogic::g"|sed "s:URL=\"http[^>]*::g" >/tmp/duptmp.xml
    fi
    
    which xmllint > /dev/null 2>&1 &&
        xmllint --format -encode UTF-8 /tmp/duptmp.xml 2>/dev/null && rm -f /tmp/duptmp.xml && return
    cat /tmp/duptmp.xml 2>/dev/null
	rm -f /tmp/duptmp.xml >/dev/null 2>&1
}

# System logging functions
logSystem() {
    ! bMakeSysLogEntry && return
    eventID=$1
    sPkgName="Package Release ID=$pkgReleaseId"
    if [ "$PMBrandStatus" = "DEBRAND" ]; then
       pkgDesc=`echo $pkgDescription|sed "s:Dell :$PMManufacturerName :g"`
    else
        pkgDesc=`echo $pkgDescription|sed "s:Dell :$PMManufacturerName :g"`
    fi
    sPackageDescription="Package Description=$pkgDesc"
    sSupportLogSpec="Support Log path=$_SUPPORT_LOG_FILE"

    # Versions are reported in the extracted message
    # sPrevVersion="Previous version=$prevVersion"
    # sNewVersion="New Version=$newVersion"

    case $eventID in
    $_DEP_HARD_ERROR | $_QUAL_HARD_ERROR)
        aMsg_0=$sPkgName
        aMsg_1=$sPackageDescription
        # aMsg[2]=$sPrevVersion
        # aMsg[3]=$sNewVersion
        aMsg_2=$sSupportLogSpec
        aMsg_3=$(msgFmt PKGLOG_MESSAGE_RETCODE_$eventID)
        ;;
    *)
        aMsg_0=$sPkgName
        aMsg_1=$sPackageDescription
        aMsg_2=$sSupportLogSpec
        aMsg_3=$(msgFmt PKGLOG_MESSAGE_RETCODE_$eventID)
        ;;
    esac
    which logger > /dev/null 2>&1 && 
        logger -p user.notice -t"$_SCRIPTNAME" ${aMsg_0} ${aMsg_1} ${aMsg_3}
}

# Update the result log entry
updateResultLogEntry() {
    iErrorCode=$1
    ! bLogging && return

    [ $iErrorCode -eq $_UPGRADE_INFO ] && iErrorCode=$_EXECUTION_SUCCESS
    case $iErrorCode in
        $_QUAL_HARD_ERROR | $_DEP_HARD_ERROR)
            if bCheckOnly; then
                logSupport SPSETUP_VALIDATION_FAIL_MSG
            else
                logSupport PKGLOG_MESSAGE_PKGFAIL
            fi
            ;;
        $_DEP_SOFT_ERROR)
            if bCheckOnly; then
                logSupport SPSETUP_VALIDATION_MSG
            else
                logSupport PKGLOG_MESSAGE_PKGFAIL
            fi
            ;;
        $_EXECUTION_UNSUCCESS)
            logSupport PKGLOG_MESSAGE_PKGFAIL
            ;;
        $_EXECUTION_SUCCESS)
            bDebugInv && return
            bCheckOnly && {
                logSupport SPSETUP_VALIDATION_MSG
                return
            }

            logSupport VERSION_UPDATE_INFO_OPEN
            printVersions >> $_SUPPORT_LOG_FILE
            logSupport VERSION_UPDATE_INFO_CLOSE
            ;;
        $_REBOOT_REQUIRED)
            logSupport VERSION_BIOSUPDATE_INFO_OPEN
            printVersions >> $_SUPPORT_LOG_FILE
            logSupport VERSION_UPDATE_INFO_CLOSE
            ;;
        $_REBOOTING_SYSTEM)
            logSupport VERSION_BIOSUPDATE_INFO_OPEN
            printVersions >> $_SUPPORT_LOG_FILE
            logSupport VERSION_UPDATE_INFO_CLOSE
            logSupport PKGLOG_MESSAGE_REBOOTING
            ;;
        *)
            ;;
    esac
}

# Final logging
endLogging() {
    iRc=$1
    ! bLogging && return

    [ $iRc -eq $_UPGRADE_INFO ] && iRc=$_EXECUTION_SUCCESS
    logSupport PKGLOG_MESSAGE_RETCODE_$iRc
    iconv -f UTF-16 -t UTF-8 $_PACKAGE_XML_FILE >/tmp/pkg.xml 
    printPackageXml /tmp/pkg.xml >> $_SUPPORT_LOG_FILE
    rm -f /tmp/pkg.xml
    date >> $_SUPPORT_LOG_FILE
    logSystem $iRc
}


# Function is the common exit point for the script.
endScript() {
    iRc=$?

    # Sleep to give customers a chance to read output on temporary terminals 
    sleep $_EXIT_DELAY
    trap '' EXIT
    # Calling UdevHal.sh script to disable/enable the scrtch pad popup window attached by iDrac
    if [ -f $_IDRAC_POPUP_TMP_FILE ]; then
	$_UDEVHAL_SHFILE $_UDEVHAL_SHFILE_ARGSENABLE > /dev/null 2>&1
	rm -f $_IDRAC_POPUP_TMP_FILE > /dev/null 2>&1
    fi
    if bLockMade && isRoot; then
        # Only the root account can create a lock file and update the support log
        # OK to delete the lock file and to perform cleanup
        endLogging $iRc
        # remove lockfile
        [ -f $_LOCK_FILE ] && rm -f $_LOCK_FILE
    fi

    # clean up tempfiles that are unique to this script execution
    [ -f $_XML_RESULT_FILE ] && rm -f $_XML_RESULT_FILE
    [ -f $_MESSAGE_RESULT_FILE ] && rm -f $_MESSAGE_RESULT_FILE
    [ -f $_VALIDATION_MESSAGE_RESULT_FILE ] && rm -f $_VALIDATION_MESSAGE_RESULT_FILE

    # Change to current directory
    cd $_CURRDIR > /dev/null 2>&1
    
    # Reboot if necessary
    if [ $iRc -eq $_REBOOTING_SYSTEM ]; then
        trap '' TERM # reboot interrupt is now valid
        which shutdown > /dev/null 2>&1 && 
            shutdown -r now && return
        which reboot > /dev/null 2>&1 && 
            reboot && return
        # Tell customer to restart manually
        # if shutdown or reboot are unavailable.
        msgFmt SPSETUP_REBOOT_MSG 
    fi

    exit $iRc
}


# Function establishes the lock file.
enterLock_lockfile() {
    _LOCK_TIMEOUT=$((3*60*60)) # 3 hours in seconds
    _LOCKUTIL=$(which lockfile 2>/dev/null)
    stat $_IC_LOCK_FILE >/dev/null 2>&1
    _OUT=$?
    if [ $_OUT -eq 0 ] ; then
	_FOUNDICLOCK=$_TRUE
    elif ! eval $_LOCKUTIL -r0 $_LOCK_FILE -l $_LOCK_TIMEOUT 2>$_MESSAGE_RESULT_FILE; then
        if [ -d $(dirname $_LOCK_FILE) ]; then
            # lockfile utility names the lock file for the concurrency message
            # otherwise lockfile utility has some other error
            if ! grep "$_LOCK_FILE" $_MESSAGE_RESULT_FILE >/dev/null; then
                cat $_MESSAGE_RESULT_FILE
                exitDUP $_DEP_HARD_ERROR
            fi
        else
            msgFmt LOCK_FOLDER_REQUIRED
            exitDUP $_DEP_HARD_ERROR
        fi
        _LOCKFILE_MODE=$(stat -c%a $_LOCK_FILE)
        # lock file must be read-only and owned by effective user ID
        [ -O $_LOCK_FILE -a $_LOCKFILE_MODE -eq 444 ] || 
            msgFmt LOCK_FILE_COMPROMISED $_LOCK_FILE
        _FOUNDLOCK=$_TRUE
    else
        _MADELOCK=$_TRUE
    fi 
    [ -f $_XML_RESULT_FILE ] && rm -f $_XML_RESULT_FILE
    [ -f $_MESSAGE_RESULT_FILE ] && rm -f $_MESSAGE_RESULT_FILE
}
# Function establishes the lock file.
enterLock_flock() {
    _LOCKUTIL=$(which flock 2>/dev/null)

    if [ ! -d $(dirname $_LOCK_FILE) ]; then
            msgFmt LOCK_FOLDER_REQUIRED
            exitDUP $_DEP_HARD_ERROR
    fi
   
    stat $_IC_LOCK_FILE > /dev/null 2>&1
    _OUT=$?
    if [ $_OUT -eq 0 ] ; then
	_FOUNDICLOCK=$_TRUE
    else
	exec 8>$_LOCK_FILE  
	if ! flock -n -e 8 2>$_MESSAGE_RESULT_FILE; then
		_LOCKFILE_MODE=$(stat -c%a $_LOCK_FILE)
		# lock file must be read-only and owned by effective user ID
		[ -O $_LOCK_FILE -a $_LOCKFILE_MODE -eq 644 ] || 
		    msgFmt LOCK_FILE_COMPROMISED $_LOCK_FILE
		_FOUNDLOCK=$_TRUE
	 else
        _MADELOCK=$_TRUE
	fi
    fi 
    [ -f $_XML_RESULT_FILE ] && rm -f $_XML_RESULT_FILE
    [ -f $_MESSAGE_RESULT_FILE ] && rm -f $_MESSAGE_RESULT_FILE
}

# Interactively asks customer for a yes or no answer.
# This is secure since 'look' is not used to obtain message strings.
# returns true if Yes, otherwise false.
bSecureContinueYes() {
    _MSG_PROMPT_CONTINUE="Continue executing? Y/N:"
    _MSG_PROMPT_WRONG_CHOICE="Wrong choice. Enter Y/N:\n"
    var=invalid
    badChoice=""
    while [ N != "${var}" -a Y != "${var}" -a n != "${var}" -a y != "${var}" ]; do
        printf "%s"  $badChoice
        read -p "$_MSG_PROMPT_CONTINUE" var
        var=$(echo ${var:-invalid} | tr '[:lower:]' '[:upper:]')
        badChoice="$_MSG_PROMPT_WRONG_CHOICE"
    done
    [ $var = Y -o $var = y ]
}

# Interactively issue messages when dependent utilities are compromised.
issueSecurityPrompt() {
    ! bDependecyWarning && return

cat <<EO_RPM_MSG >&2
rpm verify output legend:
       . verification test passed
       S file Size differs
       M Mode differs (includes permissions and file type)
       5 MD5 sum differs
       D Device major/minor number mismatch
       L readLink(2) path mismatch
       U User ownership differs
       G Group ownership differs
       T mTime differs
EO_RPM_MSG

    ! bSecureContinueYes && {
        msgSecure "Stopping DUP execution"
        exitDUP $_SECURITY_ERROR
    }
}

#PM Data Manipulation
getPMInfo()
{
	PM_FOLDER=$(mktemp dup.XXXXXX) > /dev/null 2>&1
    mkdir -p /var/log/dell/$PM_FOLDER > /dev/null 2>&1
	PM_FILE=/var/log/dell/$PM_FOLDER/PMInfo.properties
    ./dupdisneyinstall.sh install
    LD_LIBRARY_PATH=/opt/dell/dup/lib:/opt/dell/dup64/lib:$LD_LIBRARY_PATH
    export LD_LIBRARY_PATH
    ./duppmdatacollector.bin 7 >$PM_FILE 2>/dev/null
    ./dupdisneyinstall.sh uninstall
    #read the properties file
    if [ -f $PM_FILE -a -s $PM_FILE ]; then
        . $PM_FILE
		rm -rf /var/log/dell/$PM_FOLDER > /dev/null 2>&1
    else
        PMStatus=OFF
        PMBrandStatus=DEFAULT
        PMBrandName=
        PMDefaultBrandName=
        PMManufacturerName=
        PMLicense=true
        PMAllowUpdate=true
        PMAllowUpdateBIOS=true
        PMAllowUpdateIDRAC=true
        PMAllowUpdateLC=true
    fi
	
    PMDefaultBrandName=${PMDefaultBrandName:="NOT_EXISTING_DEFAULT_BRANDNAME"}
	
    #check PM License
    if [ $PMLicense = "false" ]; then
        echo "The attempted feature is not licensed to run on this system. Please contact service provider"
        exitDUP $_EXECUTION_UNSUCCESS
    fi

    if [ $PMAllowUpdate = "false" ]; then
        echo "Update is not allowed on this system. Please contact service provider"
        exitDUP $_EXECUTION_UNSUCCESS
    fi
    validateFile $_PIE_CONFIG_SH_FILE
    . $_PIE_CONFIG_SH_FILE
    echo $pkgDescription|grep "Lifecycle Controller" >/dev/null 2>/dev/null
    ret=$?
        if [ $ret = 0 ];then
          if [ $PMAllowUpdateLC = "false" ]; then
            iconv -f UTF-16 -t UTF-8 $_PACKAGE_XML_FILE >/tmp/pkg.xml 2>/dev/null
            CID=`grep componentID /tmp/pkg.xml`
            rm -f /tmp/pkg.xml
            echo $CID |grep 28897 >/dev/null 2>/dev/null
            ret=$?
            if [ $ret = 0 ];then
              echo "LC DUP Update is not allowed on this system. Please contact service provider"
              exitDUP $_EXECUTION_UNSUCCESS
            fi
          fi
        fi
    
    echo $pkgDescription|grep "iDRAC" >/dev/null 2>/dev/null
    ret=$?
    if [ $ret = 0 ];then
       if [ $PMAllowUpdateIDRAC = "false" ]; then
            echo "iDRAC DUP Update is not allowed on this system. Please contact service provider"
            exitDUP $_EXECUTION_UNSUCCESS
       fi
    fi
    echo $pkgComponentType|grep "BIOS" >/dev/null 2>/dev/null
    ret=$?
    if [ $ret = 0 ];then
      if [ $PMAllowUpdateBIOS = "false" ]; then
          echo "BIOS DUP Update is not allowed on this system. Please contact service provider"
          exitDUP $_EXECUTION_UNSUCCESS
      fi
    fi

    if [ $PMStatus = "OFF" ]; then
      #echo "DEBUG:PMStatus is off"
      PMBrandStatus='DEFAULT'
      PMManufacturerName='Dell'
      return;
    fi
    if [ $PMBrandStatus = "DEFAULT" ]; then
      #echo "DEBUG:PMBrandStatus is default"
      PMManufacturerName='Dell'
      return;
    fi
    if [ $PMBrandStatus = "DEBRAND" ]; then
      #echo "DEBUG:PMBrandStatus is debrand"
      PMManufacturerName=''
      return;
    fi
    #echo "DEBUG:PMBrandStatus is rebrand"
    return;
}
#Check dependencies for RHEL 6 X64
checkRHEL6Dependencies () {
    _rpmstatus=false

    uname -a | grep el[6-9] | grep -q x86_64
    if [ $? -eq 0 ]; then
	for rpmname in compat-libstdc++-33.i686  libxml2.i686 libstdc++.i686; do
	    rpm -q $rpmname > /dev/null 2>&1
	    if [ $? -ne 0 ]; then  
		_rpmstatus=true
	    fi
	done
    fi
    which rpm >/dev/null 2>&1
    if [ $? -ne 0 ]
    then
        _rpmstatus=false
    fi 
    if [ $_rpmstatus = true ]; then
        echo "The following packages are required for update package to run:"
        echo "   compat-libstdc++-33.i686 libstdc++.i686 libxml2.i686"
        if [ "$PMBrandStatus" = "DEBRAND" ]; then
            echo "Please check Update package User guide for instructions for installing the dependencies"
        else
            echo "Please check $PMManufacturerName Update package User guide for instructions for installing the dependencies"
        fi
	exitDUP $_EXECUTION_UNSUCCESS
    fi
}

# Function verifies the security of the dependent utilities
verifySecurity() {
     _dep=""
    _rpm_err_msg=""
    _RPM_ERRORS=""

    # Get quiet and security override options for security verification.
    # Ignore getopts errors.
    while getopts ":qn" Option ${_CMDOPTS}; do
    case $Option in
        q) _VAL_QUIET=$_TRUE
            ;;
        n) _VAL_ALLOW_INSECURE_EXEC=$_TRUE
            ;;
        *)
            ;;
    esac
    done

    res=false
    # Make sure all the dependencies are available on the system
    for dep in $@; do
	which $dep >/dev/null 2>&1 || {
	msgSecure "$0: Cannot find $dep utility on the system to execute package.\n"
	res=true
    }
    done

    if $res ;  then
	exitDUP $_EXECUTION_UNSUCCESS
    fi

    # Use rpm verification to identify security risks in dependent utilities
    # Only flag any changes in mode, user ownership, or group ownership 
    rm -f rpm_error > /dev/null 2>&1 ; touch rpm_error > /dev/null 2>&1
    _err_ix=1
    which rpm > /dev/null 2>&1
    if [ $? -eq 0 ]
    then
    for _dep in $@; do
         _fullPath=$(which $_dep)
         _rpm_str=$(rpm --verify -f $_fullPath \
            | grep $_fullPath \
            | grep -E -e ".M......|......G.|.....U..") 
        [ -z "$_rpm_str" ] || {
            echo "rpm verify failed: $_rpm_str" >> rpm_error
            _err_ix=$((_err_ix+1))
            _DEP_WARNING=$_TRUE
        }
    done
    else
        _DEP_WARNING=$_FALSE
    fi

    if bQuiet ; then
        bDependecyWarning && {
            bAllowInsecureExec || exitDUP $_SECURITY_ERROR
        }
    else # interactive:
        for _rpm_err_msg in `cat rpm_error`; do
            msgSecure "$_rpm_err_msg" >&2
        done
        if bAllowInsecureExec; then
            bDependecyWarning && msgSecure "Continuing execution"
        else
            issueSecurityPrompt
        fi
    fi
}

# Displays libstdc++ error message depending on OS architecture
cpp_error() {
    _CPP=""
    _CPP_RPM=""
    _CPP="libstdc++.so.5"
    _CPP_RPM="compat-libstdc.i686"
    _CPP_RPM_SLES="libstdc++-33-32bit"

    echo "Error while loading shared libraries: $_CPP:"
    echo "cannot open shared object file: No such file or directory."
    echo "Install the following dependencies:"
    echo "RHEL: $_CPP_RPM"
    echo "SLES: $_CPP_RPM_SLES"
    if [ "$PMBrandStatus" = "DEBRAND" ]; then
       echo "Please check Update package User guide for instructions"
    else
       echo "Please check $PMManufacturerName Update package User guide for instructions"
    fi
    echo "for installing the dependencies."
}

# Validates libstdc++ library linkage for sputility binary
validateCpp() {
    ldd $_SPUTILITY_BIN | grep "not found" >/dev/null 2>&1 && {
        cpp_error > $_MESSAGE_RESULT_FILE
        printFormattedMessage
        logSupportFormattedMessage
        exitDUP $_DEP_HARD_ERROR
    }
}


# Block execution of pure 64-bit DUP on 32-bit OS's.
verify64DUPON32Architecture() {
#    bTestFor64BitPlatforms || return
    if [ -f ./QUAL_HARD_ERROR ]; then
       echo "This package doesn't support running on 32-bit operating systems."  | fmt -w $_WIDTH
       exitDUP $_QUAL_HARD_ERROR
    fi
}

# Blocks 64-bit OS's.
# In DUP 2.0, this blocking is not in effect,
# and bTestFor64BitPlatforms returns false.
verify64Architecture() {
    bTestFor64BitPlatforms || return
    machine=$(uname -m)
    stringlen=${#machine}
    architecture=$(echo $machine | awk  -v var=$stringlen '{ string=substr($0, 5, var - 1); print machine; }' )
    if [ $architecture -eq 64 ]; then
       echo "$_DUP_PACKAGE_VERSION doesn't support 64-bit operating systems."  | fmt -w $_WIDTH
       exitDUP $_DEP_HARD_ERROR
    fi
}

# Validates file existence
validateFile() {
    if [ ! -e $1 ]; then
        msgFmt SP_FILE_OPEN_ERROR $1
        exitDUP $_DEP_HARD_ERROR
    fi
}

# Validates variable existence
validateVariable() {
    variable=$1
    value=""
    eval value=\$${1}
    if [ -z "$value" ];then
        msgFmt GENERIC_S "Variable $variable must be set by PIEConfig.sh"
        exitDUP $_DEP_HARD_ERROR
    fi
}

# Interactively asks customer for a yes or no answer
# returns true if Yes, otherwise false
bContinueYes() {
    var=invalid
    choose=$(msgFmt MSG_PROMPT_CONTINUE)
    badChoice=""
    while [ N != "${var}" -a Y != "${var}" ]; do
        printf "%s"  $badChoice
        read -p "$choose" var
        var=$(echo ${var:-invalid} | tr '[:lower:]' '[:upper:]')
        badChoice="$(msgFmt MSG_PROMPT_WRONG_CHOICE)\n"
    done
    [ $var = Y ]
}

setExecutionReboot() {
    RANDOM_NUM=`hexdump -n 2 -e '/2 "%u"' /dev/urandom`
    tmpFile=$(mktemp $_TMPFILE_PREFIX-reboot$$-${RANDOM_NUM}.XXXXXX)
    RANDOM_NUM=`hexdump -n 2 -e '/2 "%u"' /dev/urandom`
    tmpXSLFileForPackageXML=$(mktemp $_TMPFILE_PREFIX-packagexmlparsing$$-${RANDOM_NUM}.XXXXXX)

    rebootExecutionRequired=0
    cat << EO_REBOOT_XSL > $tmpFile
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<!-- Parses SVMExecution.xml files for reboot specification -->
<xsl:output method="text" indent="no" omit-xml-declaration="yes"/>
<xsl:template match="/SVMExecution">
    <xsl:value-of select="RebootRequired"/>
</xsl:template>
</xsl:stylesheet>
EO_REBOOT_XSL

    cat << EO_PACKAGE_REBOOT_XSL > $tmpXSLFileForPackageXML
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<!-- Parses SVMExecution.xml files for reboot specification -->
<xsl:output method="text" indent="no" omit-xml-declaration="yes"/>
<xsl:template match="/SoftwareComponent">
    <xsl:value-of select="@rebootRequired"/>
</xsl:template>
</xsl:stylesheet>
EO_PACKAGE_REBOOT_XSL
    which xsltproc > /dev/null 2>&1 && {
        rebootExecutionRequired=$(xsltproc $tmpFile $_XML_RESULT_FILE)
	xsltproc $tmpXSLFileForPackageXML $_PACKAGE_XML_FILE 2>/dev/null | grep -io "true" > /dev/null 2>&1 && rebootExecutionRequired=1
        rm -f $tmpFile
	rm -f $tmpXSLFileForPackageXML
        return ${rebootExecutionRequired:-0}
    }
    # else
    cat $_XML_RESULT_FILE | grep "RebootRequired>1" > /dev/null \
        && rebootExecutionRequired=1
    	iconv -f UTF-16 -t UTF-8 $_PACKAGE_XML_FILE 2>/dev/null | grep -io "rebootRequired=\"true" > /dev/null 2>&1 && rebootExecutionRequired=1
    rm -f $tmpFile
	rm -f $tmpXSLFileForPackageXML
    return ${rebootExecutionRequired:-0}
}

# Establish whether or not to reboot the system.
# Prompt for reboot answer in non-quiet mode.
processReboot() {
    iErrorCode=$1
    iRc=$_EXECUTION_SUCCESS
    if ! bSuccess $iErrorCode; then
        logSupport SPSETUP_ERROR_UNABLE_PROCEED
        return $_EXECUTION_UNSUCCESS
    fi

    if [ ${rebootRequired:=0} -eq 1 ]; then # test PIEConfig reboot flag
        iRc=$_REBOOT_REQUIRED
    else
        setExecutionReboot # test SVMExecution reboot flag
        if [ $? -eq 1 ]; then
            iRc=$_REBOOT_REQUIRED
        else
            # else check for device level reboot in execution xml
            printXml $_XML_RESULT_FILE | grep "RebootRequired value=\"1\"" > /dev/null \
                && iRc=$_REBOOT_REQUIRED
        fi
    fi

    displayDepInfo "$_INTERNAL_SOFT_DEPENDENCY"
    logDisplayDepInfo "$_INTERNAL_SOFT_DEPENDENCY"
    # Prompt for reboot if necessary
    if [ $iRc -eq $_REBOOT_REQUIRED ]; then
        if ! bQuiet; then
            msgFmt SPSETUP_REBOOT_QUESTION # reboot?
            if bContinueYes; then
                return $_REBOOTING_SYSTEM
            else
                logSupport SPSETUP_USER_CANCELLED_REBOOT
                logSupport SPSETUP_REBOOT_MSG
                msgFmt SPSETUP_REBOOT_MSG
                if [ "$iRCDepCheck" -eq "$_INTERNAL_SOFT_DEPENDENCY" ]; then
			return $_REBOOT_REQUIRED_SOFT_DEPENDENCY_NOT_MET
		else
                return $_REBOOT_REQUIRED
		fi
            fi
        else # quiet mode
            if bReboot; then
                return $_REBOOTING_SYSTEM
            else
                logSupport SPSETUP_REBOOT_MSG
                msgFmt SPSETUP_REBOOT_MSG
		if [ "$iRCDepCheck" -eq "$_INTERNAL_SOFT_DEPENDENCY" ]; then
                        return $_REBOOT_REQUIRED_SOFT_DEPENDENCY_NOT_MET
                else
                return $_REBOOT_REQUIRED
                fi
            fi
        fi
    else  # reboot not required
		msgFmt EXE_SUCCESS_UPDATE
	if [ "$iRCDepCheck" -eq "$_INTERNAL_SOFT_DEPENDENCY" ]; then
                        return $_UPDATE_SUCCESSFUL_SOFT_DEPENDENCY_NOT_MET
        else
        return $_EXECUTION_SUCCESS
	fi
    fi
}

# Uid validation 
validateUid() {
    # Check for root user privilege for execution of package
    if ! isRoot; then
      msgScript MSG_ERROR_NOROOTPRIVELEGE
      exitDUP $_DEP_HARD_ERROR
    fi
}


# Kills processes that have timed out
timeoutCmd() {
    cmd=$1
    pid1=$2
    kill $pid1 2>&1
    
    i
    i=0
    # 5 tries before we apply kill -9 (KILL force)
    while [ $i -le 5 ]; do
        ! ps -e | grep -e "^ *$pid1 " > /dev/null && return $_TRUE
        sleep 1
        i=$((i+1))
    done

    echo $cmd: trying kill -9 $pid1 >> $_SUPPORT_LOG_FILE
    kill -9 $pid1
    sleep 1
    ps -e | grep -e "^ *$pid1 " > /dev/null && {
        return $_FALSE
    }
    return $_TRUE
}

# Executes commands that require timeout waits
# Timeout duration is in seconds
_execCmd() {
    timeout=$1
    shift
    cmd=$1
    shift
    # DFCT130826 [Expedition] PV22xs DUPs make disney unresponsive
    # We ignore a bunch of signals which seems to affect the Zappa utility
    # and dataengine which it interacts with. This causes OMSA GUI to freeze. 
    # In general we should not be setting a signal mask for the PIE inventory and
    # execution commands. They will be handling the signals in the way they find best.
    # so lets reset the signal meaning for the list we ignore before invoking the
    # PIE inventory or execution command
    trap - HUP INT QUIT BUS SEGV PIPE TERM CONT STOP TSTP XCPU # 1 2 3 10 11 13 15 18 19 20 24 
    $cmd $@ &
    pid1=$!
    # lets ignore all those signals again so that we are protected against
    # abnormal termination
    trap '' HUP INT QUIT BUS SEGV PIPE TERM CONT STOP TSTP XCPU # 1 2 3 10 11 13 15 18 19 20 24 

    i=0
    while [ $i -le $timeout ]; do
        if ! ps -e | grep -e "^ *$pid1 " > /dev/null; then
            break
        fi
        sleep 1
        echo -n "." >&2
        i=$((i+1)) 
        [ $i -eq $timeout ] && {
            timeoutCmd $cmd $pid1 && return 1
            return 2
        }
    done
    return $_TRUE
}
execCmd() {
    timeout=$1
    cmd=$2
    _execCmd $@ > $_XML_RESULT_FILE
    iRc=$?
    [ $iRc -eq 1 ] && echo "$cmd: timed out after $timeout secs" >> $_SUPPORT_LOG_FILE
    [ $iRc -eq 2 ] && echo "$cmd: could not kill -9 this process" >> $_SUPPORT_LOG_FILE
    echo >&2
    return $iRc
}

# System validation 
validateSystemID() {
    supportMessage=""
    semicolon=""
    iSysID=""
    sSysID=""
    pmStatus=0
    i=1
    br=""
#---------------------
## System is supported if PIE has no system ID's

if [ -z "${systemIDs_1}" ]
then
	return $_EXECUTION_SUCCESS ;
fi
i=1
while [ $i -gt 0 ]
do
eval "var=\${brand_$i}" # i should be controlled by the script at all times. If influenced by side-effects like user input, robust validation is required.
#printf '%s\n' "$var"
if [ -z "$var" ]
then
        brand_count=$((i-1))
        break;
fi
i=$((i+1))
done

while [ $brand_count -ne 0 ]
do
        eval "var=\${systemIDs_${brand_count}}"
        for sSysID in $var ; do
            # remove leading zeros from package system ID
            sSysID=$(printf "%s" $sSysID | sed 's/^0*\(.*\)/\1/')
	    if [ -z "$sSysID" ]
    	    then
        	continue;
            fi		
            [ $sSysID = $_SYSTEM_ID ] && return $_EXECUTION_SUCCESS ;
        done
                eval "var=\${supportedModels_${brand_count}}"
                if [ -z "$var" ]; then
                        continue
                fi
        supportMessage="$var"
        brand_count=$((brand_count-1));
done
 
    if [ $PMBrandStatus = "DEBRAND" -o $PMBrandStatus = "STRING_REBRAND" -o $PMBrandStatus = "COMPLETE_REBRAND" ]; then
		supportMessage=`echo "$supportMessage" |sed "s:PowerEdge::g"|sed "s:PowerEdge ::g" |  sed "s:Power Edge ::g"|sed "s:PowerVault::g"|sed "s:PowerVault ::g"|sed "s:Power Vault ::g"|sed "s:Equallogic ::g"|sed "s:Equallogic::g"`
    pmStatus=1
    fi
    ## The variable "pmStatus" is added to print different message when the system is branded or default.
    (
        [ -n "$_PRODUCT" -a -n "$supportMessage" -a $pmStatus = 1 ] && \
            msgFmt VAL_ERROR_SYSTEM 
        [ -n "$_PRODUCT" -a -n "$supportMessage"  -a $pmStatus = 0 ] && \
            msgFmt VAL_ERROR_SYSTEM_DETAILED_2 "$_PRODUCT" "$supportMessage"
        [ -n "$_PRODUCT" -a -z "$supportMessage" ] && \
            msgFmt VAL_ERROR_SYSTEM_DETAILED "$_PRODUCT"
        [ -z "$_PRODUCT" ] && \
            msgFmt VAL_ERROR_SYSTEM
    ) > $_MESSAGE_RESULT_FILE
    printFormattedMessage
    logSupportFormattedMessage
    updateResultLogEntry $_QUAL_HARD_ERROR
    exitDUP $_QUAL_HARD_ERROR
#--------------------- 
}

# Inventory 
runInventory() {
    logSupport INST_STATUS_RUNNING_INVENTORY
    ! bXml && msgFmt INST_STATUS_RUNNING_INVENTORY

    # Run the PIE inventory command which should output XML results
    # Print and log errors, then exit if result file is empty.
    cmd="./$sInventoryCliBin $sInventoryCliArgs"
    execCmd ${sInventoryCliTimeout:-300} $cmd
    iRc=$?

    if [ $iRc -ne 0 -o ! -s $_XML_RESULT_FILE ]; then
        [ $iRc -ne 0 ] && msgFmt INVENTORY_TIMEOUT
        msgFmt EXE_FAIL_INVENTORY
        logSupport EXE_FAIL_INVENTORY
        updateResultLogEntry $_EXECUTION_UNSUCCESS
        exitDUP $_EXECUTION_UNSUCCESS
    fi

    # Move the Inventory XML results file to the inventory XML file.
    # Update inventory XML file with systemID and OS information.
    # Print and log errors, then exit if result file is empty.
    cp $_XML_RESULT_FILE $_INVENTORY_XML_FILE
    $_SPUTILITY_BIN -i $_SYSTEM_ID -b $_BIOS_VER > $_XML_RESULT_FILE
    if [ ! -s $_XML_RESULT_FILE ]; then
        msgFmt EXE_FAIL_INVENTORY
        logSupport EXE_FAIL_INVENTORY
        updateResultLogEntry $_EXECUTION_UNSUCCESS
        exitDUP $_EXECUTION_UNSUCCESS
    fi

    # Move the Inventory XML results file to the inventory XML file used in validation.
    cp $_XML_RESULT_FILE $_INVENTORY_XML_FILE

    # Dump inventory to support logs
    printXml $_INVENTORY_XML_FILE >> $_SUPPORT_LOG_FILE

    # Print inventory XML to console if debugging inventory and XML
    if bDebugInv && bXml; then
        printXml $_INVENTORY_XML_FILE
    fi

    # Verify inventory results
    # If inventory failed, print and log errors, then exit.
    $_SPUTILITY_BIN -m $_INVENTORY_XML_FILE > $_MESSAGE_RESULT_FILE
    inventoryRc=$?
    [ $inventoryRc -eq $_UPGRADE_INFO ] && inventoryRc=$_EXECUTION_SUCCESS
    if ! bSuccess $inventoryRc; then
        if [ ! -s $_MESSAGE_RESULT_FILE ]; then
            # Message result file is empty
            msgFmt EXE_FAIL_INVENTORY
            logSupport EXE_FAIL_INVENTORY
        else
            # remove BIOS component info from inventory failure message
	    RANDOM_NUM=`hexdump -n 2 -e '/2 "%u"' /dev/urandom` 
            tmpFile=$(mktemp $_TMPFILE_PREFIX-messages$$-${RANDOM_NUM}.XXXXXX)
            if [ $? -eq 0 ]; then
                if [ "$pkgComponentType" = "BIOS" ]; then
                    cat $_MESSAGE_RESULT_FILE  > $tmpFile
                else
                    cat $_MESSAGE_RESULT_FILE | grep -v "^ *BIOS *$" > $tmpFile
                fi
                cat $tmpFile > $_MESSAGE_RESULT_FILE
                rm -f $tmpFile
            else
                echo "$_SCRIPTNAME: cannot mktemp file tmpFile"
            fi
            # print and log Message result file contents
            printFormattedMessage
            logSupportFormattedMessage
        fi
        updateResultLogEntry $_EXECUTION_UNSUCCESS
        exitDUP $_EXECUTION_UNSUCCESS
    fi

    # Exit with success if debugging inventory and XML
    if bDebugInv && bXml; then
        updateResultLogEntry $_EXECUTION_SUCCESS
        exitDUP $_EXECUTION_SUCCESS
    fi
}

# Validation 
runValidation() {
    logSupport INST_STATUS_VALIDATING
    ! bXml && msgFmt INST_STATUS_VALIDATING

    # Run validation that outputs XML results to the result file
    # Print and log errors, then exit if result file is empty.
    $_SPUTILITY_BIN -v > $_XML_RESULT_FILE
    if [ ! -s $_XML_RESULT_FILE ]; then
        msgFmt EXE_FAIL_VALIDATION
        logSupport EXE_FAIL_VALIDATION
        updateResultLogEntry $_EXECUTION_UNSUCCESS
        exitDUP $_EXECUTION_UNSUCCESS
    fi

    # Verify validation results
    # If validation fails to give messages, print and log errors, then exit.
    $_SPUTILITY_BIN -m $_XML_RESULT_FILE > $_MESSAGE_RESULT_FILE
    validationRc=$?
    [ $validationRc -eq $_UPGRADE_INFO ] && validationRc=$_EXECUTION_SUCCESS
    if [ ! -s $_MESSAGE_RESULT_FILE ]; then
        msgFmt EXE_FAIL_VALIDATION
        logSupport EXE_FAIL_VALIDATION
        updateResultLogEntry $_EXECUTION_UNSUCCESS
        exitDUP $_EXECUTION_UNSUCCESS
    fi

    # Save validation message result file for use in showing execution results
    cp $_MESSAGE_RESULT_FILE $_VALIDATION_MESSAGE_RESULT_FILE 
    if bSuccess $validationRc; then
	runDepCheck "hard"
    fi
    # Exit if only checking validation.
    if bCheckOnly; then
        if bXml; then
            # Display the validation result in XML format
            printXml $_XML_RESULT_FILE
            printXml $_XML_RESULT_FILE >> $_SUPPORT_LOG_FILE
        else
            # Print and log the validation result
            printFormattedMessage
            logSupportFormattedMessage
        fi
        updateResultLogEntry $validationRc
        exitDUP $validationRc
    fi

    # If validation has hard errors, log and display them, then exit
    if ! bSuccess $validationRc; then
        printFormattedMessage
        logSupportFormattedMessage
        updateResultLogEntry $validationRc
        exitDUP $validationRc
    fi

    # Always log soft errors or upgrade info
    # No prompts to continue in silent mode
    if bQuiet; then
        # Soft errors require a force to continue.
        if [ $validationRc -eq $_DEP_SOFT_ERROR ]; then
            printFormattedMessage
            logSupportFormattedMessage

            # If no force of any kind, return a soft error.
            if ! bForce && ! bTForce; then
                msgFmt MSG_SUGGEST_FORCE_SILENT_MODE
                logSupport MSG_SUGGEST_FORCE_SILENT_MODE
                updateResultLogEntry $validationRc
                exitDUP $validationRc
            fi

            # Get more details about the comparison between the target and the package.
            _TARGET_COMPARE_STATE=$(grep $_TARGET_COMPARE_ID $_MESSAGE_RESULT_FILE | \
                sed 's/.*= *\(.*\) *$/\1/')

            # Possible version compare states.
            _TCOMPARE_STATE_ANY_LOWER=1         # one or more current version < package
            _TCOMPARE_STATE_ALL_EQUAL=2         # all current vesion == package
            _TCOMPARE_STATE_ALL_GREATER=3       # all current version > package
            _TCOMPARE_STATE_EQUAL_AND_GREATER=4 # all current version >= package
            _TCOMPARE_STATE_ERROR=-1            # should never happen

            # Non-tiered force.  Force only for an "ALL_EQUAL" case if the target
            # is not firmware/BIOS, 
            # otherwise, do not apply the update and report a soft error.
            if bForce; then
                if [ $_TARGET_COMPARE_STATE -eq $_TCOMPARE_STATE_ALL_EQUAL ]; then
                    _COMPONENT_TYPES=$(grep $_COMPONENT_TYPE_ID $_MESSAGE_RESULT_FILE | \
                        sed 's/.*= *\(.*\) *$/\1/')
                    echo $_COMPONENT_TYPES | egrep "BIOS|FRMW" > /dev/null 2>&1 &&
                        _DO_EXECUTE_UPDATE=$_FALSE
                fi
            fi

            # Tiered force
            if bTForce; then
                case $_FORCE_LEVEL in
                0)  # level 0, force equal
                    if [ $_TARGET_COMPARE_STATE -ne $_TCOMPARE_STATE_ALL_EQUAL ]; then
                        msgFmt MSG_SUGGEST_FORCE_SILENT_MODE
                        logSupport MSG_SUGGEST_FORCE_SILENT_MODE
                        exitDUP $validationRc
                    fi
                    ;;
                1)  # level 1, force greater
                    if [ $_TARGET_COMPARE_STATE -ne $_TCOMPARE_STATE_ALL_GREATER ]; then
                        msgFmt MSG_SUGGEST_FORCE_SILENT_MODE
                        logSupport MSG_SUGGEST_FORCE_SILENT_MODE
                        exitDUP $validationRc
                    fi
                    ;;
                2)  # level 2, force equal or greater
                    if [ $_TARGET_COMPARE_STATE -eq $_TCOMPARE_STATE_ALL_GREATER \
                    -o $_TARGET_COMPARE_STATE -eq $_TCOMPARE_STATE_EQUAL_AND_GREATER \
                    -o $_TARGET_COMPARE_STATE -eq $_TCOMPARE_STATE_ALL_EQUAL ]; then
                        nop=1
                    else
                        msgFmt MSG_SUGGEST_FORCE_SILENT_MODE
                        logSupport MSG_SUGGEST_FORCE_SILENT_MODE
                        exitDUP $validationRc
                    fi
                    ;;
                esac
            fi
        fi
        # End of soft error processing.

        # Silent successful upgrades do not require a force to continue.
        # Show Validation result.
        if [ $_EXECUTION_SUCCESS -eq $validationRc ]; then
            printFormattedMessage
            logSupportFormattedMessage
        fi
    # Prompt user for continuation if not in silent mode
    else
        # Show Validation result before querying for continuation.
        printFormattedMessage
        logSupportFormattedMessage
        if ! bContinueYes; then
            msgFmt UPDATE_CANCELLED
            logSupport UPDATE_CANCELLED
            [ $validationRc -ne $_DEP_SOFT_ERROR ] && validationRc=$_DEP_SOFT_ERROR
            updateResultLogEntry $validationRc
            exitDUP $validationRc
        else
            # User elected to force execution over a soft error
            if [ $validationRc -eq $_DEP_SOFT_ERROR ]; then
                msgFmt UPDATE_FORCED
                logSupport UPDATE_FORCED
                # set _VAL_FORCE variable to true so that when we 
                # execute we know to use force - DFCT130411
                _VAL_FORCE=$_TRUE
            fi
        fi
    fi
}

displayDepInfo() {
	if [ "$1" -eq "$_INTERNAL_SOFT_DEPENDENCY" ]; then
	   if [ "$iRCDepCheck" -eq "$_INTERNAL_SOFT_DEPENDENCY" ]; then
	      msgSecure "$_SOFTDEP_CHECK_INFO"
	   fi
	elif [ "$1" -eq "$_INTERNAL_HARD_DEPENDENCY" ]; then
	   if [ "$iRCDepCheck" -eq "$_INTERNAL_HARD_DEPENDENCY" ]; then
              msgSecure "$_HARDDEP_CHECK_INFO"
           fi
	fi
}

runDepCheck() {
	LD_LIBRARY_PATH=/opt/dell/dup/lib:/opt/dell/dup64/lib:$LD_LIBRARY_PATH
	export LD_LIBRARY_PATH
	if [ "$1" = "hard"  ]; then
		_HARDDEP_CHECK_INFO="$(./LinuxDepWrapperLinux.bin $1 2>$_DEP_ERROR_TMP_FILE )"
		iRCDepCheck=$?
	elif [ "$1" = "soft" ]; then
		_SOFTDEP_CHECK_INFO="$(./LinuxDepWrapperLinux.bin $1 2>$_DEP_ERROR_TMP_FILE)" 
		iRCDepCheck=$?
	fi
}
# Execution 
runExecution() {
    logSupport INST_STATUS_EXECUTING
    msgFmt INST_STATUS_EXECUTING
    vendorName=$(echo $PMManufacturerName|tr "[:lower:]" "[:upper:]")
    # Warn about interference to updates
    if [ "$PMBrandStatus" = "DEBRAND" ]; then
      msgFmt EXE_UPDATE_WARNING ""
    else
      msgFmt EXE_UPDATE_WARNING "$vendorName "
    fi
    logSupport EXE_UPDATE_WARNING

    # Execution results are written to the system log
    _MAKE_SYSLOG_ENTRY=$_TRUE

    # Run the PIE execution command which should output XML results
    # Print and log errors, then exit if result file is empty.
    cmd="./$sExecutionCliBin $sExecutionCliArgs"
    cmdForce="./$sExecutionCliBin $sExecutionCliForceArgs"

    # Executing update
    #   Execute only if _DO_EXECUTE_UPDATE is true.
    #   This boolean is set to "false" under force conditions:
    #     - a forced an update in a same-version situation, and
    #     - the target device is firmware or BIOS.
    iRc=0
    if [ $_DO_EXECUTE_UPDATE -eq $_TRUE ]; then
        if bForce; then
            execCmd ${sExecutionCliTimeout:-1200} $cmdForce
        else
            execCmd ${sExecutionCliTimeout:-1200} $cmd
        fi
        iRc=$?
    # Execution was not necessary
    else
        msgFmt PKGLOG_MESSAGE_EXECUTION_UNNECESSARY
        logSupport PKGLOG_MESSAGE_EXECUTION_UNNECESSARY
        exitDUP $_EXECUTION_SUCCESS
    fi

    if [ $iRc -ne 0 -o ! -s $_XML_RESULT_FILE ]; then
        [ $iRc -ne 0 ] && msgFmt EXECUTION_TIMEOUT
        msgFmt GENERIC_FAIL_UPDATE
        logSupport GENERIC_FAIL_UPDATE
        updateResultLogEntry $_EXECUTION_UNSUCCESS
        exitDUP $_EXECUTION_UNSUCCESS
    fi

    # Verify execution results
    # If execution failed, print and log errors, then exit.
    $_SPUTILITY_BIN -m $_XML_RESULT_FILE > $_MESSAGE_RESULT_FILE
    executionRc=$?
    [ $executionRc -eq $_UPGRADE_INFO ] && executionRc=$_EXECUTION_SUCCESS
    # All unsuccessful execution returns map to EXECUTION_UNSUCCESS
    if ! bExecutionSuccess $executionRc; then
        executionRc=$_EXECUTION_UNSUCCESS
        if [ ! -s $_MESSAGE_RESULT_FILE ]; then
            msgFmt GENERIC_FAIL_UPDATE
            logSupport GENERIC_FAIL_UPDATE
        else
            printXml $_XML_RESULT_FILE >> $_SUPPORT_LOG_FILE
            printUpdateErrors
            logSupportUpdateErrors
        fi
        updateResultLogEntry $executionRc
        exitDUP $executionRc
    else
	runDepCheck "soft"
    fi
    # Display update information obtained in the update.xml
    if ! bQuiet; then
	    #BITS051505,xslrproc present, display using new behaviour else old
	    which xsltproc > /dev/null 2>&1
	    if [ $? -eq 0 ]
	    then
       # printUpdateErrors is a misnomer, this if for success
       printUpdateErrors 
	    else
	       # extract overallResult from top of Message result file
	       overallResult=$(head -2 $_MESSAGE_RESULT_FILE)
	       echo "$overallResult" | fmt -s -w $_WIDTH
	    fi
    fi
    # log the information, logSupportUpdateErrors is a misnomer, this if for success
    logSupportUpdateErrors

    # Prompt for reboot if necessary
    processReboot $executionRc
    executionRc=$?
    updateResultLogEntry $executionRc
    return $executionRc
}

getBuildVersion() {
    echo "$_DUP_PACKAGE_VERSION (BLD_${_DUP_BUILD_NUMBER})"
}

# Print version and release notes
printReleaseNotes() {
    bFound=0
    sSysID=""
    br=""
#------------
i=1
while [ $i -gt 0 ]
do
eval "var=\${brand_$i}" # i should be controlled by the script at all times. If influenced by side-effects like user input, robust validation is required.
#printf '%s\n' "$var"
if [ -z "$var" ]
then
        brand_count=$((i-1))
        break;
fi
i=$((i+1))
done
while [ $brand_count -ne 0 ]
do
        eval "var=\${systemIDs_${brand_count}}"
        for sSysID in $var 
        do
            # remove leading zeros from package system ID
            sSysID=$(printf "%s" $sSysID | sed 's/^0*\(.*\)/\1/')
	    if [ -z "$sSysID" ]
	    then
		continue;
	    fi	
            [ $sSysID = $_SYSTEM_ID ] && {
		         bFound=1
                  	 break;
            }
        done
        brand_count=$((brand_count-1));
done

#------------
    #PM Changes
    if [ "$PMBrandStatus" = "DEBRAND" ]; then
       PKGNAME=`echo "$_DUP_PACKAGE_NAME $(getBuildVersion)" | sed "s:Dell ::g"`
    else
       PKGNAME=`echo "$_DUP_PACKAGE_NAME $(getBuildVersion)" | sed "s:Dell :$PMManufacturerName :g"`
    fi
    echo "$PKGNAME"
    echo $_DUP_PACKAGE_COPYRIGHT
    if [ "$PMBrandStatus" = "DEFAULT" ]; then
       cat $_VERSION_FILE 
       return;
    fi
     if [ $bFound -eq 0 ]; then
         if [ "$PMBrandStatus" = "DEBRAND" ]; then
	       cat $_VERSION_FILE | sed "s:Dell Update Package:Update Package:g"|sed "s:Dell Server:Server:g"|sed "s:as Dell programs:as package utility programs:g" |sed "s: by contacting Dell at 1-800-WWW-DELL.:.:g"|sed "s:[Dd]ell::g"|sed "s:Power Edge::g"| sed "s:PowerEdge::g"| sed -e "/Supported System(s):/,/Description:/c\Supported System(s):\n$PMBrandName\n\nDescription:"| sed "s:Power Edge::g"| sed "s:PowerEdge::g"|sed "s:Power Vault::g"|sed "s:PowerVault::g" | sed "s:Equallogic::g"|sed "s:$PMDefaultBrandName::g"  | sed "s:\( *at *\)\? \"support.dell.com\"::g" 2>/dev/null
		else
		   cat $_VERSION_FILE | sed "s:Dell Update Package:$PMManufacturerName Update Package:g"|sed "s: by contacting Dell at 1-800-WWW-DELL.:.:g"|sed "s:[Dd]ell:$PMManufacturerName:g"|sed "s:Power Edge::g"| sed "s:PowerEdge::g"|sed -e "/Supported System(s):/,/Description:/c\Supported System(s):\n$PMBrandName\n\nDescription:"| sed "s:PowerEdge::g"|sed "s:Power Edge::g"|sed "s:Power Vault::g"|sed "s:PowerVault::g"| sed "s:Equallogic::g"|sed "s:$PMDefaultBrandName:$PMBrandName:g" | sed "s:\( *at *\)\? \"support.dell.com\"::g" 2>/dev/null
		fi
        return;
    fi
    #cat $_VERSION_FILE 2>/dev/null
    #PM Changes
    if [ "$PMBrandStatus" = "DEBRAND" ]; then
       cat $_VERSION_FILE | sed "s:Dell Update Package:Update Package:g"|sed "s:Dell Server:Server:g"|sed "s:as Dell programs:as package utility programs:g" |sed "s: by contacting Dell at 1-800-WWW-DELL.:.:g"|sed "s:[Dd]ell::g"|sed "s:PowerEdge::g"| sed "s:Power Edge::g"| sed -e "/Supported System(s):/,/Description:/c\Supported System(s):\n$PMBrandName\n\nDescription:"| sed "s:PowerEdge::g"| sed "s:Power Edge::g"| sed "s:Power Vault::g"|sed "s:PowerVault::g" | sed "s:Equallogic::g"|sed "s:$PMDefaultBrandName::g"  | sed "s:\( *at *\)\? \"support.dell.com\"::g" 2>/dev/null
    else
       cat $_VERSION_FILE | sed "s:Dell Update Package:$PMManufacturerName Update Package:g"|sed "s: by contacting Dell at 1-800-WWW-DELL.:.:g"|sed "s:[Dd]ell:$PMManufacturerName:g"|sed "s:Power Edge::g"| sed "s:PowerEdge::g"|sed -e "/Supported System(s):/,/Description:/c\Supported System(s):\n$PMBrandName\n\nDescription:"|sed "s:PowerEdge::g"|sed "s:Power Edge::g"|sed "s:Power Vault::g"|sed "s:PowerVault::g"| sed "s:Equallogic::g"|sed "s:$PMDefaultBrandName:$PMBrandName:g" | sed "s:\( *at *\)\? \"support.dell.com\"::g" 2>/dev/null
    fi
} 

# Print release notes using 'more'.
# Called when errors exist with the use of 'less'.
printReleaseNotesWithMore() {
    msgFmt SPSETUP_VERSION_LESS_WARNING
    if bContinueYes; then
        printReleaseNotes | fmt -s -w $_WIDTH | more
        return
    else
        exitDUP 0
    fi
} 

# Prompts used for DUP information Viewing
printReleaseNotesWithLess() {
    # do not execute 'less' in a background process
    tty --quiet || return

    which less >/dev/null
    if [ $? != 0 ]; then
        printReleaseNotesWithMore
    fi
    _DUP_TASK="continue DUP exec"
    bVersion && _DUP_TASK="exit DUP"
     _DUP_LESS_PROMPT=$(_msg SPSETUP_VERSION_PROMPT "${_DUP_TASK}")

    _DUP_LESS_PROMPT_INFO=$(_msg SPSETUP_VERSION_PROMPT_INFO)
    _DUP_LESS_PROMPT_WAIT=$(_msg SPSETUP_VERSION_PROMPT_WAIT)
    _DUP_LESS_PROMPT_SIZE=$(echo $_DUP_LESS_PROMPT | wc -c)
    _DUP_LESS_PROMPT_SIZE_N=$_DUP_LESS_PROMPT_SIZE
    _DUP_LESS_PROMPT_SIZE_N=$((_DUP_LESS_PROMPT_SIZE_N+25))
    _DUP_VIEW_PROMPT="$_DUP_LESS_PROMPT; .?ltlines %lt-%lb?L/%L. ?e(END) :?pB%pB\%..%t"
    _DUP_LESS_INFO="$_DUP_LESS_PROMPT_INFO; .?ltlines %lt-%lb?L/%L. ?e(END) :?pB%pB\%..%t"

    _PREFERRED_WIDTH=$_DUP_LESS_PROMPT_SIZE_N
    if [ ${_WIDTH} -lt $_PREFERRED_WIDTH ] 
    then
        msgFmt SPSETUP_VERSION_PROMPT_WARNING $_PREFERRED_WIDTH
        if ! bContinueYes; then
            exitDUP 0
        fi
    fi
    export LESS="-P$_DUP_VIEW_PROMPT\$-Pw$_DUP_LESS_PROMPT_WAIT\$-P=$_DUP_LESS_INFO"

    # Print release notes to file so that the less pB% directive 
    # can calculate the percent of text viewed
    RANDOM_NUM=`hexdump -n 2 -e '/2 "%u"' /dev/urandom`
    tmpFile=$(mktemp $_TMPFILE_PREFIX-releaseNotes$$-${RANDOM_NUM}.XXXXXX)
    if [ $? -eq 0 ]; then
        printReleaseNotes | fmt -s -w $_WIDTH > $tmpFile
        less $tmpFile 2>/dev/null
        if [ $? != 0 ]; then
            rm -f $tmpFile
            printReleaseNotesWithMore
        fi
        rm -f $tmpFile
    else
        printReleaseNotes | fmt -s -w $_WIDTH | less 2>/dev/null
    fi
}

#########################################################################
# Main() Script
#########################################################################

# Verify security dependencies before setting signal traps
verify64DUPON32Architecture
if [ -f ./checkCompatDependencies ]
then
	 checkRHEL6Dependencies
fi
# Calling UdevHal.sh script to disable/enable the scrtch pad popup window attached by iDrac
if [ -x $_UDEVHAL_SHFILE ]; then
	$_UDEVHAL_SHFILE $_UDEVHAL_SHFILE_ARGSDISABLE > /dev/null 2>&1
	touch $_IDRAC_POPUP_TMP_FILE > /dev/null 2>&1
fi
cat $_PIE_CONFIG_SH_FILE | sed 's/\[/_/g' | sed 's/\]=/=/g' > $_PIE_CONFIG_SH_FILE.tmp
cp $_PIE_CONFIG_SH_FILE.tmp $_PIE_CONFIG_SH_FILE

RANDOM_NUM=`hexdump -n 2 -e '/2 "%u"' /dev/urandom`
_MESSAGE_RESULT_FILE=$(mktemp $_TMPFILE_PREFIX-MessageResult$$-${RANDOM_NUM}.XXXXXX) || {
    echo "$_SCRIPTNAME: cannot mktemp file _MESSAGE_RESULT_FILE"
    exit $_DEP_HARD_ERROR
}
# Establish a lock or test for concurrency
which lockfile > /dev/null 2>&1
if [ $? = 0 ]; then 
    enterLock_lockfile
else
    enterLock_flock
fi 
# Lock out other DUP executions - only one DUP can write to support log at a time.
bLockFound && {
    msgFmt SPSETUP_ERROR_MULTIPROCESS
    exitDUP $_DEP_HARD_ERROR
}
bICLockFound && {
    msgFmt INVENTORY_COLLECTOR_RUNNING_MSG 
    exitDUP $_DEP_HARD_ERROR
}
getPMInfo
# verifySecurity "sed stty cut fmt stat look tail rm mkdir mktemp chmod ls basename dirname sleep iconv"

# Trap exit and kill for cleanup
trap 'endScript' EXIT KILL  # 0 9
# Ignore abnormal exit conditions
trap '' HUP INT QUIT BUS SEGV PIPE TERM CONT STOP TSTP XCPU # 1 2 3 10 11 13 15 18 19 20 24 

_SCRIPTNAME=`basename $0`
_SCRIPT_WORK_DIR=`dirname $0`

# stty requires a terminal to get the window width
tty -s && _WIDTH=$(stty size | cut -f2 -d' ')
if [ $_WIDTH = 0 ]
then
_WIDTH=80
fi

# Change to working directory
cd $_SCRIPT_WORK_DIR > /dev/null 2>&1

# result output files - constant file names
_XML_RESULT_FILE=$(mktemp $_TMPFILE_PREFIX-XmlResult$$-${RANDOM_NUM}.XXXXXX) || {
    echo "$_SCRIPTNAME: cannot mktemp file _XML_RESULT_FILE"
    exitDUP $_DEP_HARD_ERROR
}
_VALIDATION_MESSAGE_RESULT_FILE=$(mktemp $_TMPFILE_PREFIX-ValMessageResult$$-${RANDOM_NUM}.XXXXXX) || {
    echo "$_SCRIPTNAME: cannot mktemp file _VALIDATION_MESSAGE_RESULT_FILE"
    exitDUP $_DEP_HARD_ERROR
}

# Prepare Message handling with formatted output
if [ ! -f $_SCRIPT_WORK_DIR/$_MESSAGE_FILE ]; then
    echo "Message file is invalid: $_MESSAGE_FILE" | fmt -s -w $_WIDTH
    exitDUP $_DEP_HARD_ERROR
fi

# parse options
_COMMAND_LINE="$@"
_SHORT_OPTS="vhdxfqncr"
_LONG_OPTS="\
version,\
help,\
buildVer,\
debuginv,\
xml,\
force:"

set -- $(echo "$@" | sed 's/?/h/g')
_OPTOUT=""
_GETOPT_RC=""
getopt -T > /dev/null 2>&1
_GETOPT_RC=$?
if [ $_GETOPT_RC -eq 4 ]; then
    # Enhanced version of getopt is available
    # to parse and report its option parsing errors.
    _OPTOUT=$(getopt -l $_LONG_OPTS $_SHORT_OPTS "$@" 2>&1 >/dev/null)
    [ $? -ne 0 ] && {
        msgFmt GENERIC_S "$(echo "$_SCRIPTNAME:$_OPTOUT" | sed 's,getopt:,,g')"
        exitDUP $_DEP_HARD_ERROR
    }
    set -- $(getopt -u -l $_LONG_OPTS $_SHORT_OPTS "$@")
fi

while [ -n "$1" ]; do
    case $1 in
    -v | --version)  # Display Version Information
        _VAL_VERSION=$_TRUE
        ;;
    -h | "-?" | --help) # Display help
        cat $_HELP_FILE | \
            egrep -v "^\+|(\+)" | \
            fmt -s -w $_WIDTH
        exitDUP $_EXECUTION_SUCCESS
        ;;
    --buildVer)  # Display build number
        echo "$_DUP_PACKAGE_NAME $(getBuildVersion)"
        exitDUP $_EXECUTION_SUCCESS
        ;;
    -d | --debuginv)  # Debug Inventory processing
        _VAL_DEBUGINV=$_TRUE
        ;;
    -x | --xml)  # Log XML Debug Information
        _VAL_XML=$_TRUE
        ;;
    -f)  # Forces package updates that do not pass validation
        _VAL_FORCE=$_TRUE
        ;;
    --force) # Tiered force
        _FORCE_LEVEL="$2"
        [ -z "$2" ] && {
            msgFmt GENERIC_S "Please provide a force level = 0, 1, or 2"
            exitDUP $_DEP_HARD_ERROR
        }
        [ $_FORCE_LEVEL -ne 0 -a $_FORCE_LEVEL -ne 1 -a $_FORCE_LEVEL -ne 2 ] && {
            msgFmt GENERIC_S "Please provide a force level = 0, 1, or 2"
            exitDUP $_DEP_HARD_ERROR
        }
        shift 1
        ;;
    -q)  # Quiet non-interactive processing
        _VAL_QUIET=$_TRUE
        ;;
    -n)  # Continue with insecure execution
        _VAL_ALLOW_INSECURE_EXEC=$_TRUE
        ;;
    -c)  # Check validation without package update
        _VAL_CHECK_ONLY=$_TRUE
        ;;
    -r)  # Reboot required
        _VAL_REBOOT=$_TRUE
        ;;
    *)  # Default case - invalid option
        # If only old un-enhanced version of getopt is available,
        # this script must report option parsing errors.
        getopt -T > /dev/null 2>&1
        _GETOPT_RC=$?
        if [ $_GETOPT_RC -ne 4 ]; then
            echo "$(_msg MSG_ERROR_INVALID_OPTION) $1" | fmt -s -w $_WIDTH
            shift 1
            msgFmt MSG_SUGGESTION_HELP
            exitDUP $_DEP_HARD_ERROR
        else
            break
        fi
        ;;
    esac
    shift 1
done

# Open an xterm window for interactive processing 
# if the DUP is executed interactively (not quiet) and is not in a terminal.
! bQuiet && {
    bNoTerminal && exec xterm +ls -e $0 $@
}
# discover system info via the getSystemId utility
_SYSTEM_INFO="$(./getSystemId)"
_PRODUCT=$(printf "%s"  "$_SYSTEM_INFO" | grep "Product Name" | sed 's/.*\w: *\(.*\)/\1/')
if [ $PMBrandStatus = "DEBRAND" ]; then
    _PRODUCT=''
fi
if [ $PMBrandStatus = "STRING_REBRAND" -o $PMBrandStatus = "COMPLETE_REBRAND" ]; then
   _PRODUCT=$PMBrandName
   PMManufacturerName=$(printf "%s"  "$_SYSTEM_INFO" | grep "Vendor" | grep -v "error" | sed 's/.*\w: *\(.*\)/\1/' | sed "s: Inc.::g" | sed "s: Inc::g")
fi
_SYSTEM_ID=$(printf "%s"  "$_SYSTEM_INFO" | grep "System" | sed 's/.*0*[xX]0*\(.*\)/\1/')
_BIOS_VER=$(printf "%s"  "$_SYSTEM_INFO" | grep "BIOS" | sed 's/.*\w: *\(.*\)/\1/')

# Display version and exit for -v or --version option.
bVersion && {
    if bQuiet; then
        printReleaseNotes | fmt -s -w $_WIDTH
    else
        printReleaseNotesWithLess
    fi
    exitDUP $_EXECUTION_SUCCESS
}

# -xml option without -debuginv or -c is invalid
if bXml && !(bDebugInv || bCheckOnly); then
    msgFmt SPSETUP_INVALID_XML_OPTION
    exitDUP $_DEP_HARD_ERROR
fi

# -debuginv without -xml is invalid
if bDebugInv && ! bXml; then
    msgFmt SPSETUP_INVALID_DEBUGINV_OPTION
    exitDUP $_DEP_HARD_ERROR
fi

# User must be root from this point on - only root can write to support logs.
validateUid

## Show version info unless using silent option
bQuiet || printReleaseNotesWithLess


# Start logging.
# package.xml, PIEConfig.sh must be present for logging and DUP execution.
validateFile $_PACKAGE_XML_FILE
validateFile $_PIE_CONFIG_SH_FILE

# get PIE configuration variables
. $_PIE_CONFIG_SH_FILE

# Create the log dirs and set names for the package and support logs.
_IS_LOGGING=$_TRUE
[ -d $_COMPLETE_LOG_DIR ] || mkdir -p $_COMPLETE_LOG_DIR

validateVariable pkgReleaseId
validateVariable pkgDescription
_PACKAGE_LOG_FILE=$_LOG_DIR/${pkgReleaseId}.txt
if [ ! -e $_DUPLOGNAME ]; then
_SUPPORT_LOG_FILE=$_COMPLETE_LOG_DIR/${pkgReleaseId}.log
else
	. $_DUPLOGNAME
	_SUPPORT_LOG_FILE=$_COMPLETE_LOG_DIR/$_DUPNAMENOEXT.log		
fi

## Determine whether or not to overwrite support log
_SUPPORT_LOG_SIZE=0
[ -f $_SUPPORT_LOG_FILE ] && {
     var_temp=`cat $_SUPPORT_LOG_FILE |wc -c | sed 's/ //g'` 
     _SUPPORT_LOG_SIZE=$((${var_temp}+${_SUPPORT_LOG_SIZE}))
}
# remove file if the size is over the limit
if [ $_SUPPORT_LOG_SIZE -gt $_SUPPORT_LOG_SIZE_LIMIT ]
then
    rm -f $_SUPPORT_LOG_FILE
fi
# Write initial support log entries
logSupport PKGLOG_MESSAGE_HEADER "" "$_COMMAND_LINE" "$(date)"
logSupport GENERIC_S ""
logSupport PKGLOG_MESSAGE_RELEASEID "${pkgReleaseId}"
logSupport GENERIC_S ""
logSupport PKGLOG_MESSAGE_PKGVERSION "$(getBuildVersion)"
logSupport GENERIC_S ""

# background processes must be executed in quiet mode
if ! bQuiet; then
    tty --quiet ||  {
        msgFmt SPSETUP_INVALID_NON_QUIET_OPTION
        logSupport SPSETUP_INVALID_NON_QUIET_OPTION
        exitDUP $_DEP_HARD_ERROR
    }
fi

validateCpp
# Validate the system, run Inventory, Validation and Execution
validateSystemID
touch smaldebuglog > /dev/null 2>&1
runInventory

FILE_LIST=""
if [ ! -f $_DONOTMOVE -a -d $_PAYLOAD_DIR ]; then
    echo "MovePayload: Entry" >> $_SUPPORT_LOG_FILE
    echo "MovePayload: Processing payload directory." >> $_SUPPORT_LOG_FILE
    ls ${_PAYLOAD_DIR} | while read File; do
        Fname=`basename "${File}"`
        if [ -f "${Fname}" ]; then
	    echo "MovePayload: Exiting." >> $_SUPPORT_LOG_FILE
	    msgFmt SPSETUP_PAYLOAD_MOVE_FAILED
	    logSupport SPSETUP_PAYLOAD_MOVE_FAILED
	    exitDUP ${EXECUTION_UNSUCCESS}
        else
	    echo "MovePayload: Moving ${Fname}" >> $_SUPPORT_LOG_FILE
            mv "${_PAYLOAD_DIR}/${Fname}" "${Fname}"
        fi
    done
    rm -rf $PAYLOAD_DIR
    echo "MovePayload: Exiting." >> $_SUPPORT_LOG_FILE
else
	if [ ! -d $_PAYLOAD_DIR ]; then
		echo "MovePayload: payload directory not found. Nothing to do." >> $_SUPPORT_LOG_FILE
	fi
    echo "MovePayload: Exiting." >> $_SUPPORT_LOG_FILE	
fi

runValidation
runExecution
exitDUP $?
